import argparse
import os
import shutil
import sys


def _check_exist(start):
    if not start:
        start = os.getcwd() if len(sys.argv) == 1 else os.path.join(os.getcwd(), sys.argv[1])

    if not os.path.exists(start):
        print(f'目录 {start} 不存在, 请检查.')
        exit(-1)

    print(f'目录 {start} :')
    return start


def walk_files(res: list, start):
    for i in os.listdir(start):
        path = os.path.join(start, i)
        if os.path.isdir(path):
            walk_files(res, path)
        else:
            res.append(path)


def flatten_dir(start=None):
    start = _check_exist(start)
    start = os.path.abspath(start)

    all = []
    walk_files(all, start)

    start_len = len(start)
    for i in all:
        dst = i[start_len + 1:]
        dst = dst.replace(os.sep, '-')
        dst = os.path.join(start, dst)
        os.rename(i, dst)
        print(f"移动: {dst}")
    for i in os.listdir(start):
        path = os.path.join(start, i)
        if os.path.isdir(path):
            print(f"删除: {path}")
            shutil.rmtree(path)


def count_dir(start=None):
    start = _check_exist(start)
    nums = {
        'file': 0,
        'dir': 0
    }

    def _walk(p):
        for i in os.listdir(p):
            path = os.path.join(p, i)
            if os.path.isdir(path):
                nums['dir'] += 1
                _walk(path)
            else:
                nums['file'] += 1

    _walk(start)
    print(f'共有{nums["dir"]}个子目录.')
    print(f'共有{nums["file"]}个文件.')


def licsber():
    print('Hello, Licsber.')


def memobird():
    from licsber.notice.memobird import send_text_message

    parser = argparse.ArgumentParser(description='发送信息给咕咕机.')
    parser.add_argument('ak')
    parser.add_argument('device_id')
    parser.add_argument('text', nargs='?', default='Hello from Licsber.')
    args = parser.parse_args()

    send_text_message(args.ak, args.device_id, args.text)



import queue
import threading
import time

import requests

from .session import get_session


def _get_content(session: requests.Session, url: str, timeout: int):
    res = session.get(url, timeout=timeout)
    if res.status_code != 200:
        raise Exception('Res code error.')
    return res.content


def _loop(session: requests.Session, req_queue: queue.Queue, res: list, timeout: int):
    if not session:
        session = get_session()

    while not req_queue.empty():
        time.sleep(0.1)
        url = req_queue.get()

        try:
            content = _get_content(session, url, timeout)
        except ...:
            print(url + ' Read time out.')
            req_queue.put(url)
            req_queue.task_done()
            continue

        res.append((url, content))
        req_queue.task_done()


def mul_get_content(urls: list[str],
                    workers_num: int = 16,
                    session: requests.Session = None,
                    timeout: int = 3) -> list[(str, bytes)]:
    """
    多线程发送GET请求，获取URL内容。
    常用于批量爬取页面，下载图片等用途。
    :param urls: 下载链接列表。
    :param workers_num: 开启线程数。
    :param session: 是否使用统一的session。
    :param timeout: 默认超时等待时间。
    :return: (url, content)形式的列表。
    """
    req_queue = queue.Queue()
    for i in urls:
        req_queue.put(i)

    res = []
    for i in range(workers_num):
        th = threading.Thread(target=_loop, args=(session, req_queue, res, timeout))
        th.start()
    req_queue.join()
    return res
